@@ -19,18 +19,19 @@
 #ifndef KQUESTION_H
 #define KQUESTION_H
 
-
-#include <qtextview.h>
+#include <qhbox.h>
+#include <qtextedit.h>
 #include <qlabel.h>
 
 class QTimer;
-class QProgressBar;
+class KProgress;
 
 /**Question view
  * The Widget that is used to display the Question.
  *@author Javier Campos Morales
  */
-class KQuestion : public QFrame  {
+class KQuestion : public QHBox
+{
     Q_OBJECT
 
 public:
@@ -66,11 +67,11 @@ private:
     // Private attributes
 
     /** Main View */
-    QTextView *_view;
+    QTextEdit *_view;
     /** Main Picture */
     QLabel *_picture;
     QTimer *_countdownTimer;
-    QProgressBar *_countdownWidget;
+    KProgress *_countdownWidget;
     int _currentCount;
     int _totalCount;
 };
