@@ -12,8 +12,8 @@
 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+// 02110-1301, USA.
 
 #include "lists.h"
 
@@ -345,6 +345,7 @@ bool MacroList::loadNew( const QDomElement& docelem, std::vector<Macro*>& ret, c
 //  if ( ! sok ) return false;
 
   int unnamedindex = 1;
+  QString tmp;
 
   for ( QDomElement macroelem = docelem.firstChild().toElement();
         ! macroelem.isNull(); macroelem = macroelem.nextSibling().toElement() )
@@ -361,7 +362,7 @@ bool MacroList::loadNew( const QDomElement& docelem, std::vector<Macro*>& ret, c
       else if ( dataelem.tagName() == "Description" )
         description = dataelem.text();
       else if ( dataelem.tagName() == "Construction" )
-        hierarchy = new ObjectHierarchy( dataelem );
+        hierarchy = ObjectHierarchy::buildSafeObjectHierarchy( dataelem, tmp );
       else if ( dataelem.tagName() == "ActionName" )
         actionname = dataelem.text().latin1();
       else if ( dataelem.tagName() == "IconFileName" )
