@@ -12,8 +12,8 @@
 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+// 02110-1301, USA.
 
 #include "object_hierarchy.h"
 
@@ -67,10 +67,10 @@ class PushStackNode
 {
   ObjectImp* mimp;
 public:
-  PushStackNode( ObjectImp* imp ) : mimp( imp ) {};
+  PushStackNode( ObjectImp* imp ) : mimp( imp ) {}
   ~PushStackNode();
 
-  const ObjectImp* imp() const { return mimp; };
+  const ObjectImp* imp() const { return mimp; }
 
   int id() const;
   Node* copy() const;
@@ -121,12 +121,12 @@ class ApplyTypeNode
   std::vector<int> mparents;
 public:
   ApplyTypeNode( const ObjectType* type, const std::vector<int>& parents )
-    : mtype( type ), mparents( parents ) {};
+    : mtype( type ), mparents( parents ) {}
   ~ApplyTypeNode();
   Node* copy() const;
 
-  const ObjectType* type() const { return mtype; };
-  const std::vector<int>& parents() const { return mparents; };
+  const ObjectType* type() const { return mtype; }
+  const std::vector<int>& parents() const { return mparents; }
 
   int id() const;
   void apply( std::vector<const ObjectImp*>& stack,
@@ -194,14 +194,14 @@ public:
   // don't want to ever save this value, since we cannot guarantee it
   // remains consistent if we add properties some place..
   FetchPropertyNode( const int parent, const QCString& name, const int propid = -1 )
-    : mpropid( propid ), mparent( parent ), mname( name ) {};
+    : mpropid( propid ), mparent( parent ), mname( name ) {}
   ~FetchPropertyNode();
   Node* copy() const;
 
   void checkDependsOnGiven( std::vector<bool>& dependsstack, int loc ) const;
   void checkArgumentsUsed( std::vector<bool>& usedstack ) const;
-  int parent() const { return mparent; };
-  const QCString& propinternalname() const { return mname; };
+  int parent() const { return mparent; }
+  const QCString& propinternalname() const { return mname; }
 
   int id() const;
   void apply( std::vector<const ObjectImp*>& stack,
@@ -468,9 +468,22 @@ void ObjectHierarchy::serialize( QDomElement& parent, QDomDocument& doc ) const
   };
 }
 
-ObjectHierarchy::ObjectHierarchy( const QDomElement& parent )
+ObjectHierarchy::ObjectHierarchy()
   : mnumberofargs( 0 ), mnumberofresults( 0 )
 {
+}
+
+ObjectHierarchy* ObjectHierarchy::buildSafeObjectHierarchy( const QDomElement& parent, QString& error )
+{
+#define KIG_GENERIC_PARSE_ERROR \
+  { \
+    error = i18n( "An error was encountered at line %1 in file %2." ) \
+            .arg( __LINE__ ).arg( __FILE__ ); \
+    return 0; \
+  }
+
+  ObjectHierarchy* obhi = new ObjectHierarchy();
+
   bool ok = true;
   QString tmp;
   QDomElement e = parent.firstChild().toElement();
@@ -480,28 +493,28 @@ ObjectHierarchy::ObjectHierarchy( const QDomElement& parent )
 
     tmp = e.attribute( "id" );
     uint id = tmp.toInt( &ok );
-    assert( ok );
+    if ( !ok ) KIG_GENERIC_PARSE_ERROR;
 
-    mnumberofargs = kMax( id, mnumberofargs );
+    obhi->mnumberofargs = kMax( id, obhi->mnumberofargs );
 
     tmp = e.attribute( "requirement" );
     const ObjectImpType* req = ObjectImpType::typeFromInternalName( tmp.latin1() );
     if ( req == 0 ) req = ObjectImp::stype(); // sucks, i know..
-    margrequirements.resize( mnumberofargs, ObjectImp::stype() );
-    musetexts.resize( mnumberofargs, "" );
-    mselectstatements.resize( mnumberofargs, "" );
-    margrequirements[id - 1] = req;
-    musetexts[id - 1] = req->selectStatement();
+    obhi->margrequirements.resize( obhi->mnumberofargs, ObjectImp::stype() );
+    obhi->musetexts.resize( obhi->mnumberofargs, "" );
+    obhi->mselectstatements.resize( obhi->mnumberofargs, "" );
+    obhi->margrequirements[id - 1] = req;
+    obhi->musetexts[id - 1] = req->selectStatement();
     QDomElement esub = e.firstChild().toElement();
     for ( ; !esub.isNull(); esub = esub.nextSibling().toElement() )
     {
       if ( esub.tagName() == "UseText" )
       {
-        musetexts[id - 1] = esub.text().latin1();
+        obhi->musetexts[id - 1] = esub.text().latin1();
       }
       else if ( esub.tagName() == "SelectStatement" )
       {
-        mselectstatements[id - 1] = esub.text().latin1();
+        obhi->mselectstatements[id - 1] = esub.text().latin1();
       }
       else
       {
@@ -512,11 +525,11 @@ ObjectHierarchy::ObjectHierarchy( const QDomElement& parent )
   for (; !e.isNull(); e = e.nextSibling().toElement() )
   {
     bool result = e.tagName() == "result";
-    if ( result ) ++mnumberofresults;
+    if ( result ) ++obhi->mnumberofresults;
 
     tmp = e.attribute( "id" );
     int id = tmp.toInt( &ok );
-    assert( ok );
+    if ( !ok ) KIG_GENERIC_PARSE_ERROR;
 
     tmp = e.attribute( "action" );
     Node* newnode = 0;
@@ -525,25 +538,24 @@ ObjectHierarchy::ObjectHierarchy( const QDomElement& parent )
       // ApplyTypeNode
       QCString typen = e.attribute( "type" ).latin1();
       const ObjectType* type = ObjectTypeFactory::instance()->find( typen );
+      if ( ! type )
+      {
+        error = i18n( "This Kig file uses an object of type \"%1\", "
+                      "which this Kig version does not support."
+                      "Perhaps you have compiled Kig without support "
+                      "for this object type,"
+                      "or perhaps you are using an older Kig version." ).arg( typen );
+        return 0;
+      }
 
-
-
-
-      // BUG: what if the type doesn't exist ?
-
-
-
-
-
-      assert( type );
       std::vector<int> parents;
       for ( QDomNode p = e.firstChild(); !p.isNull(); p = p.nextSibling() )
       {
         QDomElement q = p.toElement();
-        assert( !q.isNull() ); // see above
-        assert ( q.tagName() == "arg" );
+        if ( q.isNull() ) KIG_GENERIC_PARSE_ERROR; // see above
+        if ( q.tagName() != "arg" ) KIG_GENERIC_PARSE_ERROR;
         int pid = q.text().toInt(&ok );
-        assert( ok );
+        if ( !ok ) KIG_GENERIC_PARSE_ERROR;
         parents.push_back( pid - 1 );
       };
       newnode = new ApplyTypeNode( type, parents );
@@ -554,21 +566,25 @@ ObjectHierarchy::ObjectHierarchy( const QDomElement& parent )
       QCString propname = e.attribute( "property" ).latin1();
       QDomElement arge = e.firstChild().toElement();
       int parent = arge.text().toInt( &ok );
-      assert( ok );
+      if ( !ok ) KIG_GENERIC_PARSE_ERROR;
       newnode = new FetchPropertyNode( parent - 1, propname );
     }
     else
     {
       // PushStackNode
-      assert( e.attribute( "action" ) == "push" );
+      if ( e.attribute( "action" ) != "push" ) KIG_GENERIC_PARSE_ERROR;
       QString typen = e.attribute( "type" );
-      assert ( !typen.isNull() );
-      ObjectImp* imp = ObjectImpFactory::instance()->deserialize( typen, e );
+      if ( typen.isNull() ) KIG_GENERIC_PARSE_ERROR;
+      ObjectImp* imp = ObjectImpFactory::instance()->deserialize( typen, e, error );
+      if ( ( ! imp ) && !error.isEmpty() ) return 0;
       newnode = new PushStackNode( imp );
     };
-    mnodes.resize( kMax( size_t(id - mnumberofargs), mnodes.size() ) );
-    mnodes[id - mnumberofargs - 1] = newnode;
+    obhi->mnodes.resize( kMax( size_t(id - obhi->mnumberofargs), obhi->mnodes.size() ) );
+    obhi->mnodes[id - obhi->mnumberofargs - 1] = newnode;
   };
+
+  // if we are here, all went fine
+  return obhi;
 }
 
 ArgsParser ObjectHierarchy::argParser() const
