@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 #endif
 
@@ -48,17 +48,12 @@ void ISPoll(void *);
 void handleExposure(void *);
 void connectCCD(void);
 void getBasicData(void);
-void initDataChannel(void);
-void waitForData(int rp, int wp);
-void updateDataChannel(void *p);
 void uploadFile(const char* filename);
-int  findPort(void);
 int  writeFITS(const char* filename, char errmsg[]);
 int  findcam(flidomain_t domain);
 int  setImageArea(char errmsg[]);
 int  manageDefaults(char errmsg[]);
 int  grabImage(void);
-int  findPort(void);
 int  checkPowerS(ISwitchVectorProperty *sp);
 int  checkPowerN(INumberVectorProperty *np);
 int  checkPowerT(ITextVectorProperty *tp);
@@ -122,22 +117,12 @@ int  expose;
 unsigned short  *img;
 } img_t;
 
-typedef struct {
- int rp;				/* Read pipe */
- int wp;				/* Write pipe */
- int streamReady;			/* Can we write to data stream? */
-} client_t;
-
-client_t *INDIClients;			/* INDI clients */
-static int DataPort;			/* Data Port */
-static int streamTimerID;		/* Stream ID */
-static unsigned int nclients;		/* # of clients using the binary stream */
+/*static int streamTimerID;		 Stream ID */
 
 static flidev_t fli_dev;
 static cam_t *FLICam;
 static img_t *FLIImg;
 static int portSwitchIndex;
-static int imageCount; 
 
 long int Domains[] = { FLIDOMAIN_USB, FLIDOMAIN_SERIAL, FLIDOMAIN_PARALLEL_PORT,  FLIDOMAIN_INET };
 
@@ -145,51 +130,48 @@ long int Domains[] = { FLIDOMAIN_USB, FLIDOMAIN_SERIAL, FLIDOMAIN_PARALLEL_PORT,
 
 /* Connect/Disconnect */
 static ISwitch PowerS[]          	= {{"CONNECT" , "Connect" , ISS_OFF, 0, 0},{"DISCONNECT", "Disconnect", ISS_ON, 0, 0}};
-static ISwitchVectorProperty PowerSP	= { mydev, "CONNECTION" , "Connection", COMM_GROUP, IP_RW, ISR_1OFMANY, 60, IPS_IDLE, PowerS, NARRAY(PowerS), 0, 0};
+static ISwitchVectorProperty PowerSP	= { mydev, "CONNECTION" , "Connection", COMM_GROUP, IP_RW, ISR_1OFMANY, 60, IPS_IDLE, PowerS, NARRAY(PowerS), "", 0};
 
 /* Types of Ports */
 static ISwitch PortS[]           	= {{"USB", "", ISS_ON, 0, 0}, {"Serial", "", ISS_OFF, 0, 0}, {"Parallel", "", ISS_OFF, 0, 0}, {"INet", "", ISS_OFF, 0, 0}};
-static ISwitchVectorProperty PortSP	= { mydev, "Port Type", "", COMM_GROUP, IP_RW, ISR_1OFMANY, 0, IPS_IDLE, PortS, NARRAY(PortS), 0, 0};
+static ISwitchVectorProperty PortSP	= { mydev, "Port Type", "", COMM_GROUP, IP_RW, ISR_1OFMANY, 0, IPS_IDLE, PortS, NARRAY(PortS), "", 0};
 
 /* Types of Frames */
 static ISwitch FrameTypeS[]		= { {"FRAME_LIGHT", "Light", ISS_ON, 0, 0}, {"FRAME_BIAS", "Bias", ISS_OFF, 0, 0}, {"FRAME_DARK", "Dark", ISS_OFF, 0, 0}, {"FRAME_FLAT", "Flat Field", ISS_OFF, 0, 0}};
-static ISwitchVectorProperty FrameTypeSP = { mydev, "FRAME_TYPE", "Frame Type", EXPOSE_GROUP, IP_RW, ISR_1OFMANY, 0, IPS_IDLE, FrameTypeS, NARRAY(FrameTypeS), 0, 0};
+static ISwitchVectorProperty FrameTypeSP = { mydev, "CCD_FRAME_TYPE", "Frame Type", EXPOSE_GROUP, IP_RW, ISR_1OFMANY, 0, IPS_IDLE, FrameTypeS, NARRAY(FrameTypeS), "", 0};
 
 /* Frame coordinates. Full frame is default */
 static INumber FrameN[]          	= {
- { "X", "", "%.0f", 0.,     MAX_PIXELS, 1., 0., 0, 0, 0},
- { "Y", "", "%.0f", 0.,     MAX_PIXELS, 1., 0., 0, 0, 0},
- { "Width", "", "%.0f", 0., MAX_PIXELS, 1., 0., 0, 0, 0},
- { "Height", "", "%.0f",0., MAX_PIXELS, 1., 0., 0, 0, 0}};
- static INumberVectorProperty FrameNP = { mydev, "FRAME", "Frame", IMAGE_GROUP, IP_RW, 60, IPS_IDLE, FrameN, NARRAY(FrameN), 0, 0};
+ { "X", "X", "%.0f", 0.,     MAX_PIXELS, 1., 0., 0, 0, 0},
+ { "Y", "Y", "%.0f", 0.,     MAX_PIXELS, 1., 0., 0, 0, 0},
+ { "WIDTH", "Width", "%.0f", 0., MAX_PIXELS, 1., 0., 0, 0, 0},
+ { "HEIGHT", "Height", "%.0f",0., MAX_PIXELS, 1., 0., 0, 0, 0}};
+ static INumberVectorProperty FrameNP = { mydev, "CCD_FRAME", "Frame", IMAGE_GROUP, IP_RW, 60, IPS_IDLE, FrameN, NARRAY(FrameN), "", 0};
  
  /* Binning */ 
  static INumber BinningN[]       = {
  { "HOR_BIN", "X", "%0.f", 1., MAX_X_BIN, 1., 1., 0, 0, 0},
  { "VER_BIN", "Y", "%0.f", 1., MAX_Y_BIN, 1., 1., 0, 0, 0}};
- static INumberVectorProperty BinningNP = { mydev, "BINNING", "Binning", IMAGE_GROUP, IP_RW, 60, IPS_IDLE, BinningN, NARRAY(BinningN), 0, 0};
+ static INumberVectorProperty BinningNP = { mydev, "CCD_BINNING", "Binning", IMAGE_GROUP, IP_RW, 60, IPS_IDLE, BinningN, NARRAY(BinningN), "", 0};
  
  /* Exposure time */
-  static INumber ExposeTimeN[]    = {{ "EXPOSE_S", "Duration (s)", "%5.2f", 0., 36000., .5, 1., 0, 0, 0}};
-  static INumberVectorProperty ExposeTimeNP = { mydev, "EXPOSE_DURATION", "Expose", EXPOSE_GROUP, IP_RW, 60, IPS_IDLE, ExposeTimeN, NARRAY(ExposeTimeN), 0, 0};
+  static INumber ExposeTimeN[]    = {{ "EXPOSE_DURATION", "Duration (s)", "%5.2f", 0., 36000., .5, 1., 0, 0, 0}};
+  static INumberVectorProperty ExposeTimeNP = { mydev, "CCD_EXPOSE_DURATION", "Expose", EXPOSE_GROUP, IP_RW, 60, IPS_IDLE, ExposeTimeN, NARRAY(ExposeTimeN), "", 0};
  
   /* Temperature control */
  static INumber TemperatureN[]	  = { {"TEMPERATURE", "Temperature", "%+06.2f", MIN_CCD_TEMP, MAX_CCD_TEMP, .2, 0., 0, 0, 0}};
- static INumberVectorProperty TemperatureNP = { mydev, "CCD_TEMPERATURE", "Temperature (C)", EXPOSE_GROUP, IP_RW, 60, IPS_IDLE, TemperatureN, NARRAY(TemperatureN), 0, 0};
+ static INumberVectorProperty TemperatureNP = { mydev, "CCD_TEMPERATURE", "Temperature (C)", EXPOSE_GROUP, IP_RW, 60, IPS_IDLE, TemperatureN, NARRAY(TemperatureN), "", 0};
  
- /* Expose progress */
- static INumber ExposeProgressN[] = { {"Time left", "", "%.0f", 0., 0., 0., 0., 0, 0, 0} };
- static INumberVectorProperty ExposeProgressNP  = { mydev, "Expose Progress (s)", "", EXPOSE_GROUP, IP_RO, 0, IPS_IDLE, ExposeProgressN, NARRAY(ExposeProgressN), 0, 0};
-  
- /* Pixel size (µm) */
+  /* Pixel size (µm) */
 static INumber PixelSizeN[] 	= {
 	{ "Width", "", "%.0f", 0. , 0., 0., 0., 0, 0, 0},
 	{ "Height", "", "%.0f", 0. , 0., 0., 0., 0, 0, 0}};
-static INumberVectorProperty PixelSizeNP = { mydev, "Pixel Size (µm)", "", DATA_GROUP, IP_RO, 0, IPS_IDLE, PixelSizeN, NARRAY(PixelSizeN), 0, 0};
+static INumberVectorProperty PixelSizeNP = { mydev, "Pixel Size (µm)", "", DATA_GROUP, IP_RO, 0, IPS_IDLE, PixelSizeN, NARRAY(PixelSizeN), "", 0};
 
- /* Data channel */
-static INumber DataChannelN[]		= {{"CHANNEL", "Channel", "%0.f", 1024., 20000., 1., 0., 0, 0, 0}};
-static INumberVectorProperty DataChannelNP={ mydev, "DATA_CHANNEL", "Data Channel", DATA_GROUP, IP_RO, 0, IPS_IDLE, DataChannelN, NARRAY(DataChannelN), 0, 0};
+/* BLOB for sending image */
+static IBLOB imageB = {"CCD1", "Feed", "", 0, 0, 0, 0, 0, 0, 0};
+static IBLOBVectorProperty imageBP = {mydev, "Video", "Video", COMM_GROUP,
+  IP_RO, 0, IPS_IDLE, &imageB, 1, "", 0};
 
 /* send client definitions of all properties */
 void ISInit()
@@ -211,11 +193,8 @@ void ISInit()
    return;
  }
  
- imageCount = 0;
- streamTimerID = -1;
- INDIClients = NULL;
- nclients    = 0;
- 
+ IEAddTimer (POLLMS, ISPoll, NULL);
+
  isInit = 1;
  
 }
@@ -231,6 +210,7 @@ void ISGetProperties (const char *dev)
   /* COMM_GROUP */
   IDDefSwitch(&PowerSP, NULL);
   IDDefSwitch(&PortSP, NULL);
+  IDDefBLOB(&imageBP, NULL);
   
   /* Expose */
   IDDefSwitch(&FrameTypeSP, NULL);  
@@ -241,30 +221,11 @@ void ISGetProperties (const char *dev)
   IDDefNumber(&FrameNP, NULL);
   IDDefNumber(&BinningNP, NULL);
   
-  /* Data Group */
-  IDDefNumber (&DataChannelNP, NULL);
-  
-  INDIClients = INDIClients ? (client_t *) realloc(INDIClients, (nclients+1) * sizeof(client_t)) :
-                              (client_t *) malloc (sizeof(client_t));
-    
-  if (INDIClients)
-  {
-        INDIClients[nclients].streamReady = 0;
-	INDIClients[nclients].rp	  = 0;
-	INDIClients[nclients].wp          = 0;
-  	DataPort = findPort();
-        if (DataPort > 0)
- 		initDataChannel();
+}
 
-        /* Send the basic data to the new client if the previous client(s) are already connected. */		
-	if (PowerSP.s == IPS_OK)
-	  getBasicData();
-  }
-  
- 
-  	
-  IEAddTimer (POLLMS, ISPoll, NULL);
-  
+void ISNewBLOB (const char *dev, const char *name, int sizes[], char *blobs[], char *formats[], char *names[], int n)
+{
+  dev=dev;name=name;sizes=sizes;blobs=blobs;formats=formats;names=names;n=n;
 }
   
 void ISNewSwitch (const char *dev, const char *name, ISState *states, char *names[], int n)
@@ -697,28 +658,6 @@ void ISPoll(void *p)
 	    break;
 	 }
 	 
-	 switch (ExposeProgressNP.s)
-	 {
-	   case IPS_IDLE:
-	   case IPS_OK:
-	   	break;
-           case IPS_BUSY:
-	      ExposeProgressN[0].value--;
-	      
-	      if (ExposeProgressN[0].value > 0)
-	          IDSetNumber(&ExposeProgressNP, NULL);
-	      else
-	      {
-	        ExposeProgressNP.s = IPS_IDLE;
-		IDSetNumber(&ExposeProgressNP, NULL);
-	      }
-	      break;
-	      
-	  case IPS_ALERT:
-	     break;
-	  }
-		
-	 
 	 switch (TemperatureNP.s)
 	 {
 	   case IPS_IDLE:
@@ -804,53 +743,55 @@ int setImageArea(char errmsg[])
  N.B. No processing is done on the image */
 int grabImage()
 {
-  long err;
-  int img_size,i, fd;
-  char errmsg[ERRMSG_SIZE];
-  char filename[] = "/tmp/fitsXXXXXX";
+	long err;
+	int img_size,i, fd;
+	char errmsg[ERRMSG_SIZE];
+	char filename[] = "/tmp/fitsXXXXXX";
   
-   if ((fd = mkstemp(filename)) < 0)
-   { 
-    IDMessage(mydev, "Error making temporary filename.");
-    IDLog("Error making temporary filename.\n");
-    return -1;
-   }
-   close(fd);
+	if ((fd = mkstemp(filename)) < 0)
+	{ 
+		IDMessage(mydev, "Error making temporary filename.");
+		IDLog("Error making temporary filename.\n");
+		return -1;
+	}
+	close(fd);
      
-  img_size = FLIImg->width * FLIImg->height * sizeof(unsigned short);
+	img_size = FLIImg->width * FLIImg->height * sizeof(unsigned short);
+	
+	FLIImg->img = malloc (img_size);
   
-  FLIImg->img = malloc (img_size);
+	if (FLIImg->img == NULL)
+	{
+		IDMessage(mydev, "Not enough memory to store image.");
+		IDLog("Not enough memory to store image.\n");
+		return -1;
+	}
   
-  if (FLIImg->img == NULL)
-  {
-    IDMessage(mydev, "Not enough memory to store image.");
-    IDLog("Not enough memory to store image.\n");
-    return -1;
-  }
+	for (i=0; i < FLIImg->height ; i++)
+	{
+		if ( (err = FLIGrabRow(fli_dev, &FLIImg->img[i * FLIImg->width], FLIImg->width)))
+		{
+			free(FLIImg->img);
+			IDMessage(mydev, "FLIGrabRow() failed at row %d. %s.", i, strerror((int)-err));
+			IDLog("FLIGrabRow() failed at row %d. %s.\n", i, strerror((int)-err));
+			return -1;
+		}
+	}
   
-  for (i=0; i < FLIImg->height ; i++)
-  {
-    if ( (err = FLIGrabRow(fli_dev, &FLIImg->img[i * FLIImg->width], FLIImg->width)))
-    {
-       free(FLIImg->img);
-       IDMessage(mydev, "FLIGrabRow() failed at row %d. %s.", i, strerror((int)-err));
-       IDLog("FLIGrabRow() failed at row %d. %s.\n", i, strerror((int)-err));
-       return -1;
-    }
-  }
-  
-   /*err = (ImageFormatS[0].s == ISS_ON) ? writeFITS(FileNameT[0].text, errmsg) : writeRAW(FileNameT[0].text, errmsg);*/
-   err = writeFITS(filename, errmsg);
+	IDMessage(mydev, "Download complete.\n");
+	
+	/*err = (ImageFormatS[0].s == ISS_ON) ? writeFITS(FileNameT[0].text, errmsg) : writeRAW(FileNameT[0].text, errmsg);*/
+	err = writeFITS(filename, errmsg);
    
-   if (err)
-   {
-       free(FLIImg->img);
-       IDMessage(mydev, errmsg, NULL);
-       return -1;
-   }
-   
-  free(FLIImg->img);
-  return 0;
+	if (err)
+	{
+		free(FLIImg->img);
+		IDMessage(mydev, errmsg, NULL);
+		return -1;
+	}
+
+	free(FLIImg->img);
+	return 0;
  
 }
 
@@ -928,10 +869,9 @@ int writeFITS(const char* filename, char errmsg[])
 void uploadFile(const char* filename)
 {
    FILE * fitsFile;
-   char frameSize[FRAME_ILEN];
    unsigned char *fitsData, *compressedData;
-   int n=0,r=0;
-   unsigned int i =0, j = 0, nr = 0, frameLen;
+   int r=0;
+   unsigned int i =0, nr = 0;
    uLongf compressedBytes=0;
    uLong  totalBytes;
    struct stat stat_p; 
@@ -970,7 +910,8 @@ void uploadFile(const char* filename)
    }
    
    compressedBytes = sizeof(char) * totalBytes + totalBytes / 64 + 16 + 3;
-     
+    
+   /* #2 Compress it */ 
    r = compress2(compressedData, &compressedBytes, fitsData, totalBytes, 9);
    if (r != Z_OK)
    {
@@ -979,36 +920,13 @@ void uploadFile(const char* filename)
 	return;
    }
    
-   snprintf(frameSize, FRAME_ILEN, "<Data type='FITS' size='%ld' compsize='%ld' />", totalBytes, compressedBytes);
-   frameLen = strlen(frameSize);
-   r = 0;
-   
-     for (i=0; i < nclients; i++)
-     {
-  	if (INDIClients[i].streamReady)
-        {
-	
-	      /* #1 send frame size to client */
-	     for (j=0; j < frameLen; j+=r)
-	      r = write(INDIClients[i].wp, frameSize + j , frameLen - j);
-	      
-	     /* Send the actual file */
-             for (nr=0; nr < compressedBytes; nr+=n)
-   	     {
-	  	n = write(INDIClients[i].wp, compressedData + nr, compressedBytes - nr );
-		if (n <= 0)
-		{
-			if (nr <= 0)
-    			{
-      				IDLog("Stream error: %s\n", strerror(errno));
-      				return;
-    			}
-		        break;
-		}		
-             }
-        }
-      } 
-    
+   /* #3 Send it */
+   imageB.blob = compressedData;
+   imageB.bloblen = compressedBytes;
+   imageB.size = totalBytes;
+   strcpy(imageB.format, ".fits.z");
+   imageBP.s = IPS_OK;
+   IDSetBLOB (&imageBP, NULL);
    
    free (fitsData);   
    free (compressedData);
@@ -1270,59 +1188,66 @@ int checkPowerT(ITextVectorProperty *tp)
 
 void connectCCD()
 {
-  long err;
-  char errmsg[ERRMSG_SIZE];
- 
-  IDLog ("In ConnectCCD\n");
+	long err;
+	char errmsg[ERRMSG_SIZE];
+	
+	IDLog ("In ConnectCCD\n");
    
   /* USB by default {USB, SERIAL, PARALLEL, INET} */
-  switch (PowerS[0].s)
-  {
-    case ISS_ON:
-      IDLog("Current portSwitch is %d\n", portSwitchIndex);
-      IDLog("Attempting to find the camera in domain %ld\n", Domains[portSwitchIndex]);
-      if (findcam(Domains[portSwitchIndex]))
-      {
-	  PowerSP.s = IPS_IDLE;
-	  PowerS[0].s = ISS_OFF;
-	  PowerS[1].s = ISS_ON;
-	  IDSetSwitch(&PowerSP, "Error: no cameras were detected.");
-	  IDLog("Error: no cameras were detected.\n");
-	  return;
-      }
+	switch (PowerS[0].s)
+	{
+		case ISS_ON:
+		IDLog("Current portSwitch is %d\n", portSwitchIndex);
+		IDLog("Attempting to find the camera in domain %ld\n", Domains[portSwitchIndex]);
+		if (findcam(Domains[portSwitchIndex])) {
+			PowerSP.s = IPS_IDLE;
+			PowerS[0].s = ISS_OFF;
+			PowerS[1].s = ISS_ON;
+			IDSetSwitch(&PowerSP, "Error: no cameras were detected.");
+			IDLog("Error: no cameras were detected.\n");
+			return;
+		}
+
+		if ((err = FLIOpen(&fli_dev, FLICam->name, FLIDEVICE_CAMERA | FLICam->domain)))
+		{
+			PowerSP.s = IPS_IDLE;
+			PowerS[0].s = ISS_OFF;
+			PowerS[1].s = ISS_ON;
+			IDSetSwitch(&PowerSP, "Error: FLIOpen() failed. %s.", strerror( (int) -err));
+			IDLog("Error: FLIOpen() failed. %s.\n", strerror( (int) -err));
+			return;
+		}
       
-      if ((err = FLIOpen(&fli_dev, FLICam->name, FLIDEVICE_CAMERA | FLICam->domain)))
-      {
-          PowerSP.s = IPS_IDLE;
-	  PowerS[0].s = ISS_OFF;
-	  PowerS[1].s = ISS_ON;
-	  IDSetSwitch(&PowerSP, "Error: FLIOpen() failed. %s.", strerror( (int) -err));
-	  IDLog("Error: FLIOpen() failed. %s.\n", strerror( (int) -err));
-	  return;
-      }
+		/* Sucess! */
+		PowerS[0].s = ISS_ON;
+		PowerS[1].s = ISS_OFF;
+		PowerSP.s = IPS_OK;
+		IDSetSwitch(&PowerSP, "CCD is online. Retrieving basic data.");
+		IDLog("CCD is online. Retrieving basic data.\n");
+		getBasicData();
+		if (manageDefaults(errmsg))
+		{
+			IDMessage(mydev, errmsg, NULL);
+			IDLog("%s", errmsg);
+			return;
+		}
+
+		break;
       
-      /* Sucess! */
-      PowerS[0].s = ISS_ON;
-      PowerS[1].s = ISS_OFF;
-      PowerSP.s = IPS_OK;
-      IDSetSwitch(&PowerSP, "CCD is online. Retrieving basic data.");
-      IDLog("CCD is online. Retrieving basic data.\n");
-      getBasicData();
-      if (manageDefaults(errmsg))
-      {
-        IDMessage(mydev, errmsg, NULL);
-	IDLog("%s", errmsg);
-	return;
-      }
-      
-      break;
-      
-    case ISS_OFF:
-      PowerS[0].s = ISS_OFF;
-      PowerS[1].s = ISS_ON;
-      PowerSP.s = IPS_IDLE;
-      IDSetSwitch(&PowerSP, "CCD is offline.");
-      break;
+		case ISS_OFF:
+			PowerS[0].s = ISS_OFF;
+			PowerS[1].s = ISS_ON;
+			PowerSP.s = IPS_IDLE;
+			if ((err = FLIClose(fli_dev))) {
+				PowerSP.s = IPS_IDLE;
+				PowerS[0].s = ISS_OFF;
+				PowerS[1].s = ISS_ON;
+				IDSetSwitch(&PowerSP, "Error: FLIClose() failed. %s.", strerror( (int) -err));
+				IDLog("Error: FLIClose() failed. %s.\n", strerror( (int) -err));
+				return;
+			}
+			IDSetSwitch(&PowerSP, "CCD is offline.");
+			break;
      }
 }
 
@@ -1370,28 +1295,27 @@ int findcam(flidomain_t domain)
     
      FLICam->domain = domain;
      
-      /* Each driver handles _only_ one camera for now */
-      switch (domain)
-      {
-      case FLIDOMAIN_PARALLEL_PORT:
-	strcpy(FLICam->dname, "parallel port");
-	break;
+	switch (domain)
+	{
+		case FLIDOMAIN_PARALLEL_PORT:
+			FLICam->dname = strdup("parallel port");
+			break;
 
-      case FLIDOMAIN_USB:
-	strcpy(FLICam->dname , "USB");
-	break;
+		case FLIDOMAIN_USB:
+			FLICam->dname = strdup("USB");
+			break;
 
-      case FLIDOMAIN_SERIAL:
-	strcpy(FLICam->dname , "serial");
-	break;
+		case FLIDOMAIN_SERIAL:
+			FLICam->dname = strdup("serial");
+			break;
 
-      case FLIDOMAIN_INET:
-	strcpy (FLICam->dname , "inet");
-	break;
+		case FLIDOMAIN_INET:
+			FLICam->dname = strdup("inet");
+			break;
 
-      default:
-	strcpy(FLICam->dname , "Unknown domain");
-      }
+		default:
+			FLICam->dname = strdup("Unknown domain");
+	}
       
       FLICam->name = strdup(tmplist[0]);
       
@@ -1441,9 +1365,9 @@ FITS_HDU_LIST * create_fits_header (FITS_FILE *ofp, uint width, uint height, uin
  hdulist->naxisn[0] = width;
  hdulist->naxisn[1] = height;
  hdulist->naxisn[2] = bpp;
- hdulist->used.datamin = min();
+ hdulist->used.datamin = 1;
  hdulist->datamin = min();
- hdulist->used.datamax = max();
+ hdulist->used.datamax = 1;
  hdulist->datamax = max();
  hdulist->used.bzero = 1;
  hdulist->bzero = 0.0;
@@ -1509,251 +1433,3 @@ double max()
     
     return lmax;
 }
-
-
-void initDataChannel ()
-{
-	int pid;
-	int rp[2], wp[2];
-
-	/* new pipes */
-	if (pipe (rp) < 0) {
-	    fprintf (stderr, "%s: read pipe: %s\n", me, strerror(errno));
-	    exit(1); 
-	}
-	if (pipe (wp) < 0) {
-	    fprintf (stderr, "%s: write pipe: %s\n", me, strerror(errno));
-	    exit(1);
-	}
-
-	/* new process */
-	pid = fork();
-	if (pid < 0) {
-	    fprintf (stderr, "%s: fork: %s\n", me, strerror(errno));
-	    exit(1);
-	}
-	if (pid == 0) {
-	    /* child: listen to driver */
-	    close(wp[1]);
-	    close(rp[0]);
-	    
-	    waitForData(wp[0], rp[1]);
-	    
-	    return;
-	}
-
-	close (wp[0]);
-	close (rp[1]);
-	INDIClients[nclients].rp = rp[0];
-	INDIClients[nclients].wp = wp[1];
-	
-	nclients++;
-	
-	fprintf(stderr, "Going to listen on fd %d, and write to fd %d\n", rp[0], wp[1]);
-	updateDataChannel(NULL);
-
-}
-
-void updateDataChannel(void *p)
-{
-  char buffer[1];
-  fd_set rs;
-  unsigned int nr, i;
-  struct timeval tv;
-  tv.tv_sec  = 0;
-  tv.tv_usec = 0;
-  p=p;
-  
-  for (i=0; i < nclients; i++)
-  {
-        if (INDIClients[i].rp < 0)
-	 continue;
-	 
-  	FD_ZERO(&rs);
-  	FD_SET(INDIClients[i].rp, &rs);
-  
-  	nr = select(INDIClients[i].rp+1, &rs, NULL, NULL, &tv);
-  
-  	if (nr <= 0)
-	 continue;
-
-  	nr = read(INDIClients[i].rp, buffer, 1);
-  	if (nr > 0 && atoi(buffer) == 0)
-	{
-	        IDLog("Client %d is ready to receive stream\n", i);
-		INDIClients[i].streamReady = 1;
-	}
-	else
-	{
-		INDIClients[i].streamReady = 0;
-		INDIClients[i].rp          = -1;
-		IDLog("Lost connection with client %d\n", i);
-	}
-   }
-  
-   addTimer(1000, updateDataChannel, NULL);
-  
-}
-
-int findPort()
-{
-  struct sockaddr_in serv_socket;
-  int sfd;
-  int port = 8000;
-  int i=0;
-  
-  /* make socket endpoint */
-  if ((sfd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
-	    fprintf (stderr, "%s: socket: %s", me, strerror(errno));
-	    exit(1);
-  }
-	
-  /* bind to given port for any IP address */
-  memset (&serv_socket, 0, sizeof(serv_socket));
-  serv_socket.sin_family = AF_INET;
-  serv_socket.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
-  
-  for (i=0; i < 100; i++)
-  {
-  	serv_socket.sin_port = htons ((unsigned short)port);
-	if (bind(sfd,(struct sockaddr*)&serv_socket,sizeof(serv_socket)) < 0)
-	{
-	  	fprintf (stderr, "%s: bind: %s\n", me, strerror(errno));
-		port +=5;
-	}
-	else break;
-  }
-  
-  close(sfd);
-  if (i == 100) return -1;
-  
-  return port;
-	
-}
-		
-  
-void waitForData(int rp, int wp)
-{
-        struct sockaddr_in serv_socket;
-	struct sockaddr_in cli_socket;
-	int cli_len;
-	int sfd, lsocket;
-	int reuse = 1;
-        fd_set rs;
-	int maxfd;
-	int i, s, nr, n;
-	unsigned char buffer[PIPEBUFSIZ];
-	char dummy[8];
-	
-	/* make socket endpoint */
-	if ((sfd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
-	    fprintf (stderr, "%s: socket: %s", me, strerror(errno));
-	    exit(1);
-	}
-	
-	/* bind to given port for any IP address */
-	memset (&serv_socket, 0, sizeof(serv_socket));
-	serv_socket.sin_family = AF_INET;
-	serv_socket.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
-	
-	serv_socket.sin_port = htons ((unsigned short)DataPort);
-	if (setsockopt(sfd,SOL_SOCKET,SO_REUSEADDR,&reuse,sizeof(reuse)) < 0){
-		fprintf (stderr, "%s: setsockopt: %s", me, strerror(errno));
-	    	exit(1);
-		}
-	if (bind(sfd,(struct sockaddr*)&serv_socket,sizeof(serv_socket)) < 0){
-	    	fprintf (stderr, "%s: bind: %s\n", me, strerror(errno));
-		exit(1);
-	    	}
-	
-        /* Tell client we're ready to connect */
-	/* N.B. This only modifies the child DataChannelNP, the parent's property remains unchanged. */
-	DataChannelN[0].value = DataPort;
-        DataChannelNP.s = IPS_OK;
-        IDSetNumber(&DataChannelNP, NULL);
-	
-	/* willing to accept connections with a backlog of 1 pending */
-	if (listen (sfd, 1) < 0) {
-	    fprintf (stderr, "%s: listen: %s", me, strerror(errno));
-	    exit(1);
-	}
-
-	/* ok */
-	lsocket = sfd;
-        fprintf (stderr, "%s: listening to data port %d on fd %d\n",me,DataPort,sfd);
-
-	/* start with public contact point */
-	FD_ZERO(&rs);
-	FD_SET(lsocket, &rs);
-	maxfd = lsocket;
-
-	/* wait for action */
-	s = select (maxfd+1, &rs, NULL, NULL, NULL);
-	if (s < 0) {
-	    fprintf (stderr, "%s: select: %s\n", me, strerror(errno));
-	    exit(1);
-	}
- 
-	/* get a private connection to new client */
-	cli_len = sizeof(cli_socket);
-	s = accept (lsocket, (struct sockaddr *)&cli_socket, (socklen_t *)&cli_len);
-	if(s < 0) {
-	    fprintf (stderr, "%s: accept: %s", me, strerror(errno));
-	    exit (1);
-	}
-	
-	/* Tell the driver that this client established a connection to the data channel and is ready
-	   to receive stream data */
-	sprintf(dummy, "%d", 0);
-	write(wp, dummy, 1);
-	
-	FD_ZERO(&rs);
-	FD_SET(rp, &rs);
-	
-	while (1)
-	{
-		i = select(rp+1, &rs, NULL, NULL, NULL);
-		if (i < 0)
-		{
-	  		fprintf (stderr, "%s: select: %s\n", me, strerror(errno));
-			/* Tell driver the client has disconnected the data channel */
-			sprintf(dummy, "%d", -1);
-			write(wp, dummy, 1);
-	  		exit(1);
-		}
-		
-		nr = read(rp, buffer, PIPEBUFSIZ);
-		if (nr < 0) {
-		sprintf(dummy, "%d", -1);
-		write(wp, dummy, 1);
-	    	exit(1);
-		}
-		if (nr == 0) {
-		sprintf(dummy, "%d", -1);
-		write(wp, dummy, 1);
-		fprintf (stderr, "Client %d: EOF\n", s);
-	    	exit(1);
-		}
-		
-		for (i=0; i < nr; i+=n)
-		{
-		  n = write(s, buffer + i, nr - i);
-		  
-		  if (n <= 0)
-		  {
-		        fprintf(stderr, "nr is %d\n", nr);
-			
-			if (n < 0)
-		    		fprintf(stderr, "%s\n", strerror(errno));
-			else
-		    	        fprintf(stderr, "Short write of FITS pixels.\n");
-				
-		       sprintf(dummy, "%d", -1);
-		       write(wp, dummy, 1);
-		       exit(1);
-		  }		
-		}
-		
-	}
-
-}
