@@ -548,7 +548,7 @@ bool verbSpanish::conjugateAll()
 	good = good && conjugateSubPresente();
 	good = good && conjugateSubPasado();
 	good = good && conjugateImperativo();
-	if ("" == partizip)
+	if (partizip.isEmpty())
 	{
 		partizip = getStem();
 		switch (getEnding(good))
@@ -564,7 +564,7 @@ bool verbSpanish::conjugateAll()
 			good = false;
 		};
 	};
-	if ("" == gerund)
+	if (gerund.isEmpty())
 	{
 		gerund = getStem();
 		switch (getEnding(good))
